library(kohonen)

# Creating Self-organising maps in R
# Load the kohonen package 
require(kohonen)

data1 <- read.csv("filepath/Chicago_data.csv", check.names=FALSE)
# Normalize the data, remove fid, wetland area, and wetland volume columns 
data2 <- data1[,-1]
data3 <- data2[,-1]
data4 <- data3[,-1]
str(data4)
TO_Wetland_SOM <- scale(data4, center = TRUE, scale = TRUE)
summary(TO_Wetland_SOM)

# Set seed and define grid
set.seed(45)
som_grid <- somgrid(xdim = 4, ydim = 4, topo = "hexagonal")

# Creaate SOM Model
som_model <- som(TO_Wetland_SOM, 
                 grid=som_grid, 
                 rlen=500, 
                 alpha=c(0.05,0.01),
                 keep.data = TRUE)

#Changes plot
plot(som_model, type = "changes")
# Codes plot
plot(som_model)

#Plot neighbour distance
shadesOfGrey <- colorRampPalette(c("grey0", "grey100"))
plot(som_model, type = "dist.neighbours", palette.name=shadesOfGrey)

#Node count plot
plot(som_model, type="count", main="Node Counts", palette.name = coolBlueHotRed)

# Kohonen Heatmap creation
# LCC Code
plot(som_model, type = "property", property = 
       getCodes(som_model)[,1], 
     main=colnames(getCodes(som_model))[1], 
     palette.name=coolBlueHotRed)

# Drainage Area dam2
plot(som_model, type = "property", property = 
       getCodes(som_model)[,2], 
     main=colnames(getCodes(som_model))[2], 
     palette.name=coolBlueHotRed)

# Average Elevation
plot(som_model, type = "property", property = 
       getCodes(som_model)[,3], 
     main=colnames(getCodes(som_model))[3], 
     palette.name=coolBlueHotRed)

# Wetland Vol per Area
plot(som_model, type = "property", property = 
       getCodes(som_model)[,4], 
     main=colnames(getCodes(som_model))[4], 
     palette.name=coolBlueHotRed)

mydata <- som_model$codes[[1]]

# Visualising cluster results
## use hierarchical clustering to cluster the codebook vectors
som_cluster <- cutree(hclust(dist(mydata)), k = 4)

# define pretty_palette
pretty_palette <- c("#2ca02c","#1f77b4", "#d62728", "#ff7f0e", "#9467bd","#8c564b","#e377c2")

# plot these results:
plot(som_model, type="mapping", bgcol = pretty_palette[som_cluster], main = "Clusters")
add.cluster.boundaries(som_model, som_cluster)

# get vector with cluster value for each original data sample
cluster_assignment <- som_cluster[som_model$unit.classif]
# for each of analysis, add the assignment as a column in the original data:
data1$cluster <- cluster_assignment

# Write new csv with cluster assignment
write.csv(data1, "filepath/Chicago_result.csv")

